@@ -190,10 +190,10 @@ class CarrierRules(GenericRule):
         """ total cost of importing and exporting carrier
 
         .. math::
-            C_y^{\mathcal{C}} = \sum_{c\in\mathcal{C}}\sum_{n\in\mathcal{N}}\sum_{t\in\mathcal{T}} \\tau_t (O_{c,n,t} + O_{c,n,t}^{\mathrm{shed}\ \mathrm{demand}})
+            C_y^{\\mathcal{C}} = \\sum_{c\\in\\mathcal{C}}\\sum_{n\\in\\mathcal{N}}\\sum_{t\\in\\mathcal{T}} \\tau_t (O_{c,n,t} + O_{c,n,t}^{\\mathrm{shed}\\ \\mathrm{demand}})
 
         :math:`O_{c,n,t}`: cost of importing and exporting carrier :math:`c` at node :math:`n` and time step :math:`t`\n
-        :math:`O_{c,n,t}^{\mathrm{shed}\ \mathrm{demand}}`: cost of shedding demand of carrier :math:`c` at node :math:`n` and time step :math:`t`\n
+        :math:`O_{c,n,t}^{\\mathrm{shed}\\ \\mathrm{demand}}`: cost of shedding demand of carrier :math:`c` at node :math:`n` and time step :math:`t`\n
         :math:`\\tau_t`: duration of time step :math:`t`
 
 
@@ -212,9 +212,9 @@ class CarrierRules(GenericRule):
         """ total carbon emissions of importing and exporting carrier
 
         .. math::
-            E_y^{\mathcal{C}} = \sum_{c\in\mathcal{C}}\sum_{n\in\mathcal{N}}\sum_{t\in\mathcal{T}} \\tau_t \\theta_{c,n,t}^{\mathrm{carrier}}
+            E_y^{\\mathcal{C}} = \\sum_{c\\in\\mathcal{C}}\\sum_{n\\in\\mathcal{N}}\\sum_{t\\in\\mathcal{T}} \\tau_t \\theta_{c,n,t}^{\\mathrm{carrier}}
 
-        :math:`\\theta_{c,n,t}^{\mathrm{carrier}}`: carbon emissions of importing and exporting carrier :math:`c` at node :math:`n` and time step :math:`t`\n
+        :math:`\\theta_{c,n,t}^{\\mathrm{carrier}}`: carbon emissions of importing and exporting carrier :math:`c` at node :math:`n` and time step :math:`t`\n
         :math:`\\tau_t`: duration of time step :math:`t`
 
         """
@@ -258,13 +258,13 @@ class CarrierRules(GenericRule):
         """node- and year-dependent carrier availability to import/export from outside the system boundaries
 
         .. math::
-            \\underline{a}_{c,n,y}^\mathrm{Y} \geq \\sum_{t\\in\mathcal{T}}\\tau_t \\underline{U}_{c,n,t}
+            \\underline{a}_{c,n,y}^\\mathrm{Y} \\geq \\sum_{t\\in\\mathcal{T}}\\tau_t \\underline{U}_{c,n,t}
 
         .. math::
-            \\overline{a}_{c,n,y}^\mathrm{Y} \geq \\sum_{t\\in\mathcal{T}}\\tau_t \\overline{U}_{c,n,t}
+            \\overline{a}_{c,n,y}^\\mathrm{Y} \\geq \\sum_{t\\in\\mathcal{T}}\\tau_t \\overline{U}_{c,n,t}
 
-        :math:`\\underline{a}_{c,n,y}^\mathrm{Y}`: yearly availability of carrier :math:`c` to import at node :math:`n`\n
-        :math:`\\overline{a}_{c,n,y}^\mathrm{Y}`: yearly availability of carrier :math:`c` to export at node :math:`n`\n
+        :math:`\\underline{a}_{c,n,y}^\\mathrm{Y}`: yearly availability of carrier :math:`c` to import at node :math:`n`\n
+        :math:`\\overline{a}_{c,n,y}^\\mathrm{Y}`: yearly availability of carrier :math:`c` to export at node :math:`n`\n
         :math:`\\tau_t`: is the duration of time step :math:`t`\n
         :math:`\\underline{U}_{c,n,t}`: flow of carrier :math:`c` imported at node :math:`n` at time step :math:`t`\n
         :math:`\\overline{U}_{c,n,t}`: flow of carrier :math:`c` exported at node :math:`n` at time step :math:`t`
@@ -314,10 +314,10 @@ class CarrierRules(GenericRule):
         """ cost and limit of shedding demand of carrier
 
         .. math::
-           O_{c,n,t}^{\mathrm{shed}\ \mathrm{demand}} = D_{c,n,t} \\nu_c \n
-           D_{c,n,t} \leq d_{c,n,t}
+           O_{c,n,t}^{\\mathrm{shed}\\ \\mathrm{demand}} = D_{c,n,t} \\nu_c \n
+           D_{c,n,t} \\leq d_{c,n,t}
 
-        :math:`O_{c,n,t}^{\mathrm{shed}\ \mathrm{demand}}`: total cost of shedding demand of carrier :math:`c` at node :math:`n` and time step :math:`t`\n
+        :math:`O_{c,n,t}^{\\mathrm{shed}\\ \\mathrm{demand}}`: total cost of shedding demand of carrier :math:`c` at node :math:`n` and time step :math:`t`\n
         :math:`\\nu_c`: price to shed demand of carrier :math:`c`\n
         :math:`D_{c,n,t}`: shed demand of carrier :math:`c` at node :math:`n` and time step :math:`t`\n
         :math:`d_{c,n,t}`: demand of carrier :math:`c` at node :math:`n` and time step :math:`t`
@@ -345,9 +345,9 @@ class CarrierRules(GenericRule):
         """ carbon emissions of importing and exporting carrier
 
         .. math::
-           \\theta_{c,n,t}^{\mathrm{carrier}} = \\underline{\\epsilon_c} \\underline{U}_{c,n,t} - \\overline{\\epsilon_c} \\overline{U}_{c,n,t}
+           \\theta_{c,n,t}^{\\mathrm{carrier}} = \\underline{\\epsilon_c} \\underline{U}_{c,n,t} - \\overline{\\epsilon_c} \\overline{U}_{c,n,t}
 
-        :math:`\\theta_{c,n,t}^{\mathrm{carrier}}`: carbon emissions of importing and exporting carrier :math:`c` at node :math:`n` and time step :math:`t`\n
+        :math:`\\theta_{c,n,t}^{\\mathrm{carrier}}`: carbon emissions of importing and exporting carrier :math:`c` at node :math:`n` and time step :math:`t`\n
         :math:`\\underline{\\epsilon_c}`: carbon intensity of carrier import :math:`c`\n
         :math:`\\overline{\\epsilon_c}`: carbon intensity of carrier export :math:`c`\n
         :math:`\\underline{U}_{c,n,t}`: flow of carrier :math:`c` imported at node :math:`n` and time step :math:`t`\n
@@ -376,14 +376,14 @@ class CarrierRules(GenericRule):
 
         .. math::
             0 = -(d_{c,n,t}-D_{c,n,t})
-            + \\sum_{i\\in\mathcal{I}}(\\overline{G}_{c,i,n,t}-\\underline{G}_{c,i,n,t})
-            + \\sum_{j\\in\mathcal{J}}(\\sum_{e\\in\\underline{\mathcal{E}}}(F_{j,e,t}-F^\mathrm{l}_{j,e,t})-\\sum_{e'\\in\\overline{\mathcal{E}}}F_{j,e',t})
-            + \\sum_{k\\in\mathcal{K}}(\\overline{H}_{k,n,t}-\\underline{H}_{k,n,t})
+            + \\sum_{i\\in\\mathcal{I}}(\\overline{G}_{c,i,n,t}-\\underline{G}_{c,i,n,t})
+            + \\sum_{j\\in\\mathcal{J}}(\\sum_{e\\in\\underline{\\mathcal{E}}}(F_{j,e,t}-F^\\mathrm{l}_{j,e,t})-\\sum_{e'\\in\\overline{\\mathcal{E}}}F_{j,e',t})
+            + \\sum_{k\\in\\mathcal{K}}(\\overline{H}_{k,n,t}-\\underline{H}_{k,n,t})
             + \\underline{U}_{c,n,t} - \\overline{U}_{c,n,t}
 
         Sources of carrier :math:`c` at node :math:`n` and time step :math:`t`:\n
         :math:`\\overline{G}_{c,i,n,t}`: output flow of carrier :math:`c` from all conversion technologies :math:`i` at node :math:`n` at time step :math:`t`\n
-        :math:`F_{j,e,t}`: transported flow of carrier :math:`c` on ingoing edges :math:`e` minues the losses :math:`F^\mathrm{l}_{j,e,t})` of all transport technologies :math:`j` at time step :math:`t`\n
+        :math:`F_{j,e,t}`: transported flow of carrier :math:`c` on ingoing edges :math:`e` minues the losses :math:`F^\\mathrm{l}_{j,e,t})` of all transport technologies :math:`j` at time step :math:`t`\n
         :math:`\\overline{H}_{k,n,t}`: output flow of carrier :math:`c` from all storage technologies :math:`k` at node :math:`n` at time step :math:`t`\n
         :math:`\\underline{U}_{c,n,t}`: flow of carrier :math:`c` imported at node :math:`n` at time step :math:`t`\n
 
